// Copyright 1997 by Jon Dart.  All Rights Reserved.

#ifndef _BOARD_STATE_H
#define _BOARD_STATE_H

#include "board.h"
#include <memory.h>

// This class maintains state information about a board
// position that is needed to restore the board state
// when a move is "undone".
//
class Board_State
{
   friend class Board;
   friend class Log_Entry;
public:
   Board_State()
   {
   }

   Board_State(const Board &b) {
#ifdef _DEBUG
     old_hashcode = b.hashCode;
     old_pawn_hashcode = b.m_pawn_hash;
     old_castlestatus[White] = b.my_CastleStatus[White];
     old_castlestatus[Black] = b.my_CastleStatus[Black];
     old_enpassantsq = b.my_EnPassantSq;
     old_checkStatus = b.my_CheckStatus;
 	 old_movecount = b.move_count;
#else
     memcpy(&old_hashcode,&b.hashCode,sizeof(Board_State));
#endif
   }


   void restoreState(Board &toBoard) const {
#ifdef _DEBUG
     toBoard.hashCode = old_hashcode;
     toBoard.m_pawn_hash = old_pawn_hashcode;
     toBoard.my_CastleStatus[White] = old_castlestatus[White];
     toBoard.my_CastleStatus[Black] = old_castlestatus[Black];
     toBoard.my_EnPassantSq = old_enpassantsq;
     toBoard.my_CheckStatus = old_checkStatus;
	 toBoard.move_count = old_movecount;
#else
     memcpy(&toBoard.hashCode,&old_hashcode,sizeof(Board_State));
#endif
   }
   
private:   
	hash_t old_hashcode, old_pawn_hashcode;
	Board::CastleType old_castlestatus[2];
	Square old_enpassantsq;
    Board::CheckStatusType old_checkStatus;
    int old_movecount;
};

#endif

