// Copyright 1994, 1997 by Jon Dart.  All Rights Reserved.

// do not use _MOVE_H - conflicts with gcc library header
#ifndef _MOVE_H_
#define _MOVE_H_

#include "types.h"
#include "square.h"
#include "piece.h"

#ifndef _BOARD_STATE_H
// forward decl. of Board
class Board;
#endif
#include <iostream>

typedef uint64 Move;

enum MoveState {NewMove, Used, Mate};

union MoveUnion
{
   struct
   {
        byte my_start;
	byte my_dest;
        byte my_promotion;
        byte pieceMoved;
        byte capture;
        byte type;
        byte flag;
        byte junk;
   } contents;

   Move m;
   struct
   {
      uint32 lopart;
      uint32 hipart;
   } split;
   
   MoveUnion(const Move move)
   : m(move)
   {
   }

   MoveUnion()
   {
   }
};

enum SpecialType { Normal, KCastle, QCastle, EnPassant, Promotion };
	
extern Move CreateMove(const Board &b,Square start, Square dest, PieceType promotion );

inline Move CreateMove(Square start, Square dest, PieceType promotion,
 PieceType pieceMoved, PieceType capture, SpecialType type)
{
   MoveUnion mu;
   mu.contents.my_start = start;
   mu.contents.my_dest = dest;
   mu.contents.my_promotion = (byte)promotion;
   mu.contents.pieceMoved = (byte)pieceMoved;
   mu.contents.capture = (byte)capture;
   mu.contents.type = (byte)type;
   mu.contents.flag = NewMove;
   return mu.m;
}	

inline Move CreateMove(Square start, Square dest) 
{
   MoveUnion mu;
   mu.contents.my_start = start;
   mu.contents.my_dest = dest;
   mu.contents.my_promotion = (byte)InvalidPiece;
   return mu.m;
}

#define IsUsed(move) (((union MoveUnion*)&(move))->contents.flag == Used)

#define SetUsed(move) ((union MoveUnion*)&(move))->contents.flag = Used

#define IsMate(move) (((union MoveUnion*)&(move))->contents.flag == Mate)

#define SetMate(move) ((union MoveUnion*)&(move))->contents.flag = Mate

#define MovesEqual(move1,move2) (((union MoveUnion)move1).split.lopart \
 == ((union MoveUnion)move2).split.lopart)

#define EqualMove(move1,move2) MovesEqual(move1,move2)
                                 
#define NullMove (Move)0x0000000000077f7f
	
#define StartSquare(move) (Square)(((union MoveUnion)move).contents.my_start)

#define DestSquare(move) (Square)(((union MoveUnion)move).contents.my_dest)

#define PromoteTo(move) ((PieceType)((union MoveUnion)move).contents.my_promotion)

#define Capture(move) ((PieceType)((union MoveUnion)move).contents.capture)

#define PieceMoved(move) ((PieceType)((union MoveUnion)move).contents.pieceMoved)

#define TypeOfMove(move) ((SpecialType)((union MoveUnion)move).contents.type)

#define IsNull(move) (StartSquare(move) == InvalidSquare)
	
//#define CaptureType(move) PieceType(Capture(move))

#define IllegalMove (Move)0xffffffff

// returns a human-readable string.  CAUTION: reuses
// a static buffer for its return value
extern const char *MoveImage(Move m);
	
#endif

