// Copyright 1999 by Jon Dart.  All Rights Reserved.

#include "posnbook.h"
#include "globals.h"
#include "util.h"
#include <stdio.h>

#define NUM_ENTRIES 32768

PositionBook::PositionBook()
: pBook(NULL),
pPage(NULL),
book_file(NULL),
hFileMapping(NULL),
  open(0),newBook(0)
{                 
   const char *fileName = POSITION_BOOK_NAME;
   static char book_path[MAX_PATH];
   learn_size = NUM_ENTRIES*sizeof(Position_Info)+sizeof(Book_Header);
   int newBook = 0;
   strcpy(book_path,programPath);
   {
      // find the last part of the program pathname
#ifdef _WIN32
      char *p = strrchr(book_path,'\\');
#else
      char *p = strrchr(book_path,'/');
#endif
      if (p)
      {
         // replace it with the book file name
         strcpy(p+1,fileName);
      }
      else
         strcpy(book_path,fileName);
      book_file = bu.openFile(book_path,false);
      if (book_file == NULL)
      {
         // create new learning file
         if (theLog) theLog->write("creating position.bin\n");
         book_file = bu.createFile(book_path,learn_size
           );
         ++newBook;
      }
   }
   init(book_path,book_file);
   
}

void PositionBook::init(char *book_path,FILE_HANDLE book_file) 
{
   if (book_file != NULL)
   {
      hFileMapping = bu.createMap(book_file,false);
   }
   if (hFileMapping)
   {
      pBook = bu.map(hFileMapping,0,learn_size,false);
      if (newBook)
      {
         *((char*)pBook) = Book_Version;
      }
   }

   open = pBook != NULL;
   if (!open)
      return;

   char *p = (char*)pBook;

   if (*p != (byte)Book_Version)
   {
      char msg[128];
      sprintf(msg,"Truncating old position.bin (version %d)\n",
              (int)*p);
      cerr << msg << endl;
      bu.unmap(pBook,4096);
      bu.closeMap(hFileMapping);
      bu.closeFile(book_file);
      open = 0;
      pBook = NULL;
      book_file = bu.createFile(book_path,learn_size);
      ++newBook;
      init(book_path,book_file);
   }
}


PositionBook::~PositionBook()
{
   if (pBook)
      bu.unmap(pBook,4096);
   if (hFileMapping)
      bu.closeMap(hFileMapping);
   if (book_file)
      bu.closeFile(book_file);
}

void PositionBook::add(const Position_Info &info)
{
   hash_t index = quickmod(info.hash_code(),NUM_ENTRIES);
   char *p = (char*)pBook + sizeof(Book_Header) + index*sizeof(Position_Info);
   memcpy(p,&info,sizeof(Position_Info));
}

int PositionBook::size() const
{
   return NUM_ENTRIES; // currently fixed
}
        
Position_Info * PositionBook::lookup(const Board &board,int rep_count)
{
   Position_Info look(board,rep_count);
   hash_t index = quickmod(look.hash_code(),NUM_ENTRIES);
   Position_Info *p = 
    (Position_Info*)((char*)pBook + sizeof(Book_Header) + index*sizeof(Position_Info));
   if (p->hash_code() == look.hash_code())
   {
     return p;
   }
   else
     return NULL;
}
